#ifndef PIPELINE_H
#define PIPELINE_H

#include "vulkan/vulkan_core.h"
#include "cglm/types.h"
#include "vk_mem_alloc.h"
#include "command.h"

typedef struct GraphicsPipelineStruct {
  VkPipelineLayout       layout;
  VkPipeline             pipeline;
} GraphicsPipeline;

typedef struct ColoredRectStruct {
  vec3 pos;
  vec2 size;
  vec4 color;
} ColoredRect;

typedef struct UIUniformStruct {
  mat4 screen;
} UIUniform;

typedef struct CharStruct {
  vec3     pos;
  vec2     size;
  vec4     color;
  uint32_t code;
} Char;

typedef struct FontUniformStruct {
  uint32_t num_symbols;
  uint32_t width;
  uint32_t height;
} FontUniform;

typedef struct SymbolInfoStruct {
  uint32_t x;
  uint32_t top;
  uint32_t width;
} SymbolInfo;

typedef struct FontDataStruct {
  FontUniform info;
  SymbolInfo* symbols;
  uint32_t*   codes;
  uint32_t*   data;
} FontData;

typedef struct FontDescriptorStruct {
  VmaAllocation uniform_memory;
  VmaAllocation image_memory;
  VkBuffer uniform;
  VkImage image;
  VkImageView view;
  VkSampler sampler;
  VkDescriptorSet set;
} FontDescriptor;

typedef struct UILayerStruct {
  VkBuffer colored_rects;
  uint32_t colored_rect_count;

  VkBuffer        textured_rects;
  uint32_t        textured_rect_count;
  VkDescriptorSet textured_rect_descriptor;

  VkBuffer texts;
  uint32_t text_count;
  FontDescriptor font;
} UILayer;

struct RectBuffer {
  VkBuffer vertex;
  VkBuffer index;

  VmaAllocation vertex_memory;
  VmaAllocation index_memory;
};

typedef struct UIContextStruct {
  VkBuffer ui_descriptor_buffer;
  VmaAllocation ui_descriptor_memory;
  VkDescriptorSetLayout ui_descriptor_layout;
  VkDescriptorPool ui_descriptor_pool;
  VkDescriptorSet ui_descriptor_set;

  VkDescriptorPool font_pool;
  VkDescriptorSetLayout font_layout;
  
  struct RectBuffer ui_rect;
  GraphicsPipeline ui_pipeline_rect;
  GraphicsPipeline ui_pipeline_text;
} UIContext;

VkResult init_pipelines(VkDevice device, VmaAllocator allocator, VkExtent2D swapchain_extent, VkRenderPass render_pass, Queue transfer_queue, VkCommandPool transfer_pool, UIContext* context);

VkResult create_text_descriptor(VkDevice device, VmaAllocator allocator, VkDescriptorSetLayout layout, VkDescriptorPool pool, FontData* font, VkCommandPool transfer_pool, Queue transfer_queue, FontDescriptor* descriptor);

#endif
